import os
import sys
import time
import logging
import random
import datetime
import smtplib
from tcw.database import session, init_engine
from tcw.utils import expired_contests
from tcw.apps.contest.models import Contest
from tcw_tasks.models import Message
from sendgrid import SendGridAPIClient


# globals #
logger = logging.getLogger('tcw-tasks')
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s|%(name)s|%(levelname)s|%(message)s'
)

def main():
    uri = os.getenv('SQLALCHEMY_DATABASE_URI', None)
    if not uri:
        logger.error('Must have SQLALCHEMY_DATABASE_URI environment var')
        sys.exit(1)

    init_engine(uri)
    logger.info("STARTING")
    while True:
        finish_contests()
        time.sleep(random.randint(60,180))


def finish_contests():
    now = datetime.datetime.now()

    try:
        contests = expired_contests()
        logger.info("%d contests pending closure" % len(contests))
    except:
        logger.debug("No contests pending closure")
        return

    for c in contests:
        print(c)
        try:
            if c.attributes is None or 'winners' not in c.attributes:
                winners = c.pick_winners()
                c.attributes = {'winners': winners}

            notify_owner(c)
            logger.info("Closing contest (%s) %s" % (c.name, c.title))
            session.delete(c)
            session.commit()

        except Exception as x:
            logger.warning(x)
            session.rollback()


def notify_owner(contest):
    local = not bool(os.getenv('SENDGRID_API_KEY', 0))
    msg = Message(contest=contest).get_message(local)
    if local:
        with smtplib.SMTP('localhost', 25) as s:
            s.send_message(msg)
    else:
        client = SendGridAPIClient(os.getenv('SENDGRID_API_KEY'))
        response = client.send(msg)


if __name__ == '__main__':
    main()




TEXT_TEMPLATE = """
CONTEST EXPIRED:
  - contest name: {{ contest.title }}

STATISTICS:
  - max_entrants: {{ contest.max_entrants }}
  - number of winners: {{ contest.attributes.winners | length }}
  - sign ups: {{ contest.entrants | length }}
  - expired: {{ contest.expires.strftime('%Y-%m-%d %H:%M:%S') }} UTC

{% if contest.attributes.winners | length -%}
THE WINNERS:
  {% for winner in contest.attributes.winners -%}
  {{ loop.index }}. {{ winner }}
  {% endfor %}

It is your responsibilty to contact the winners.
Thank You!
{%- else %}
Sorry. There were no winners selected, because no one signed up.
{% endif %}

Please do not respond directly to this e-mail. The originating e-mail account is not monitored.

thecontestkitty.com
"""

HTML_TEMPLATE = """
<html>
  <head>
    <style>
      .container {
        position: relative;
        width: 100%;
        max-width: 800px;
        margin: 0 auto;
        padding: 0 20px;
        box-sizing: border-box; }
      html {
        font-size: 62.5%; }
      body {
        font-size: 1.5em;
        line-height: 1.6;
        font-weight: 400;
        font-family: "HelveticaNeue", "Helvetica Neue", Helvetica, Arial, sans-serif;
  color: #222; }
      .card {
        border: 1px solid #da70d6;
        border-radius: 5px;
        padding: 1.5em; }
      footer {
        color: #999; }
    </style>
  </head>
  <body>
    <div class="container">
      <div class="card">
        <h3>Contest Expired</h3>
        <p>
        {{ contest.title }}
        </p>

        <h3>Statistics</h3>
        <div>
            <div>max entrants = {{ contest.max_entrants }}</div>
            <div>number of winners = {{ contest.attributes.winners | length }}</div>
            <div>sign-ups = {{ contest.entrants | length }}</div>
            <div>expired = {{ contest.expires.strftime('%Y-%m-%d %H:%M:%S') }} UTC</div>
        </div>

        {% if contest.attributes.winners | length %}
        <h3>The Winners</h3>
        <div class="card">
            <ol>
              {% for winner in contest.attributes.winners -%}
              <li>{{ winner }}</li>
              {% endfor %}
            </ol>
        </div>
        <footer>
            <p>
            It is your responsibilty to contact the winners.
            Please do not respond directly to this e-mail. The originating e-mail account is not monitored.
            </p>
            <p>
            <a href="https://thecontestkitty.com">thecontestkitty.com</a>
            </p>
        </footer>
        {%- else %}
        <p>Sorry. There were no winners selected, because no one signed up.</p>
        {% endif %}
      </div>
    </div>
  </body>
</html>
"""
